@@ -1,9 +1,8 @@
-# -*- coding: latin-1 -*-
+# -*- coding: utf-8 -*-
 
 """This module encapsulates mime operations.
 """
 #=======================================================================================
-__version__ = "$Revision: 1.27 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
 
@@ -15,18 +14,21 @@ import mimetypes
 import subprocess
 import shutil
 import logging
+import io
 
 
 # GNUmed
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-import gmShellAPI, gmTools, gmCfg2
+import gmShellAPI
+import gmTools
+import gmCfg2
 
 
 _log = logging.getLogger('gm.docs')
-_log.info(__version__)
+
 #=======================================================================================
-def guess_mimetype(aFileName = None):
+def guess_mimetype(filename = None):
 	"""Guess mime type of arbitrary file.
 
 	filenames are supposed to be in Unicode
@@ -37,19 +39,19 @@ def guess_mimetype(aFileName = None):
 	try:
 		import extractor
 		xtract = extractor.Extractor()
-		props = xtract.extract(filename = aFileName)
+		props = xtract.extract(filename = filename)
 		for prop, val in props:
 			if (prop == 'mimetype') and (val != worst_case):
 				return val
 	except ImportError:
-		_log.exception('Python wrapper for libextractor not installed.')
+		_log.debug('module <extractor> (python wrapper for libextractor) not installed')
 
 	ret_code = -1
 
 	# 2) use "file" system command
 	#    -i get mime type
 	#    -b don't display a header
-	mime_guesser_cmd = u'file -i -b "%s"' % aFileName
+	mime_guesser_cmd = u'file -i -b "%s"' % filename
 	# this only works on POSIX with 'file' installed (which is standard, however)
 	# it might work on Cygwin installations
 	aPipe = os.popen(mime_guesser_cmd.encode(sys.getfilesystemencoding()), 'r')
@@ -61,12 +63,12 @@ def guess_mimetype(aFileName = None):
 		if ret_code is None:
 			_log.debug('[%s]: <%s>' % (mime_guesser_cmd, pipe_output))
 			if pipe_output not in [u'', worst_case]:
-				return pipe_output
+				return pipe_output.split(u';')[0].strip()
 		else:
 			_log.error('[%s] on %s (%s): failed with exit(%s)' % (mime_guesser_cmd, os.name, sys.platform, ret_code))
 
 	# 3) use "extract" shell level libextractor wrapper
-	mime_guesser_cmd = 'extract -p mimetype "%s"' % aFileName
+	mime_guesser_cmd = 'extract -p mimetype "%s"' % filename
 	aPipe = os.popen(mime_guesser_cmd.encode(sys.getfilesystemencoding()), 'r')
 	if aPipe is None:
 		_log.debug("cannot open pipe to [%s]" % mime_guesser_cmd)
@@ -87,11 +89,12 @@ def guess_mimetype(aFileName = None):
 	_log.info("OS level mime detection failed, falling back to built-in magic")
 
 	import gmMimeMagic
-	mime_type = gmTools.coalesce(gmMimeMagic.file(aFileName), worst_case)
+	mime_type = gmTools.coalesce(gmMimeMagic.file(filename), worst_case)
 	del gmMimeMagic
 
-	_log.debug('"%s" -> <%s>' % (aFileName, mime_type))
+	_log.debug('"%s" -> <%s>' % (filename, mime_type))
 	return mime_type
+
 #-----------------------------------------------------------------------------------
 def get_viewer_cmd(aMimeType = None, aFileName = None, aToken = None):
 	"""Return command for viewer for this mime type complete with this file"""
@@ -109,6 +112,7 @@ def get_viewer_cmd(aMimeType = None, aFileName = None, aToken = None):
 	_log.debug("<%s> viewer: [%s]" % (aMimeType, viewer))
 
 	return viewer
+
 #-----------------------------------------------------------------------------------
 def get_editor_cmd(mimetype=None, filename=None):
 
@@ -126,6 +130,7 @@ def get_editor_cmd(mimetype=None, filename=None):
 	_log.debug("<%s> editor: [%s]" % (mimetype, editor))
 
 	return editor
+
 #-----------------------------------------------------------------------------------
 def guess_ext_by_mimetype(mimetype=''):
 	"""Return file extension based on what the OS thinks a file of this mimetype should end in."""
@@ -170,6 +175,7 @@ def guess_ext_for_file(aFile=None):
 		return None
 
 	return f_ext
+
 #-----------------------------------------------------------------------------------
 _system_startfile_cmd = None
 
@@ -194,7 +200,7 @@ def _get_system_startfile_cmd(filename):
 	if _system_startfile_cmd is not None:
 		return True, _system_startfile_cmd % filename
 
-	open_cmd_candidates = ['xdg-open', 'kfmclient', 'gnome-open', 'exo-open', 'op', 'open']
+	open_cmd_candidates = [u'xdg-open', u'kfmclient', u'gnome-open', u'exo-open', u'op', u'open']
 
 	for candidate in open_cmd_candidates:
 		found, binary = gmShellAPI.detect_external_binary(binary = candidate)
@@ -206,6 +212,7 @@ def _get_system_startfile_cmd(filename):
 
 	_system_startfile_cmd = u''
 	return False, None
+
 #-----------------------------------------------------------------------------------
 def convert_file(filename=None, target_mime=None, target_filename=None, target_extension=None):
 	"""Convert file from one format into another.
@@ -244,19 +251,65 @@ def convert_file(filename=None, target_mime=None, target_filename=None, target_e
 		return False
 
 	return True
+
+#-----------------------------------------------------------------------------------
+def describe_file(filename):
+
+	base_name = u'gm-describe_file'
+
+	paths = gmTools.gmPaths()
+	local_script = os.path.join(paths.local_base_dir, '..', 'external-tools', base_name)
+
+	candidates = [ base_name, local_script ]		#, base_name + u'.bat'
+	found, binary = gmShellAPI.find_first_binary(binaries = candidates)
+	if not found:
+		#binary = base_name# + r'.bat'
+		_log.debug(u'cannot find <%s(.bat)>', base_name)
+		return (False, _(u'<%s(.bat)> not found') % base_name)
+
+	desc_fname = gmTools.get_unique_filename()
+
+	cmd_line = [
+		binary,
+		filename,
+		desc_fname
+	]
+	_log.debug('describing: %s', cmd_line)
+	try:
+		gm_describe = subprocess.Popen(cmd_line)
+	except OSError:
+		_log.debug('cannot run <%s>', binary)
+		return (False, _(u'problem with <%s>') % binary)
+
+	gm_describe.communicate()
+	if gm_describe.returncode != 0:
+		_log.error('<%s> returned [%s], failed to convert', binary, gm_describe.returncode)
+		return (False, _(u'problem with <%s>') % binary)
+
+	try:
+		desc_file = io.open(desc_fname, mode = 'rt', encoding = 'utf8', errors = 'replace')
+	except IOError:
+		_log.exception('cannot open [%s]', desc_fname)
+		return (False, _(u'problem with <%s>') % binary)
+
+	desc = u''.join(desc_file.readlines())
+	desc_file.close()
+	return (True, desc)
+
 #-----------------------------------------------------------------------------------
 def call_viewer_on_file(aFile = None, block=None):
 	"""Try to find an appropriate viewer with all tricks and call it.
 
 	block: try to detach from viewer or not, None means to use mailcap default
 	"""
-	# does this file exist, actually ?
-	try:
-		open(aFile).close()
-	except:
-		_log.exception('cannot read [%s]', aFile)
-		msg = _('[%s] is not a readable file') % aFile
-		return False, msg
+	if not os.path.isdir(aFile):
+		# is the file accessible at all ?
+		try:
+			open(aFile).close()
+		except:
+			_log.exception('cannot read [%s]', aFile)
+			msg = _('[%s] is not a readable file') % aFile
+			return False, msg
 
 	# try to detect any of the UNIX openers
 	found, startfile_cmd = _get_system_startfile_cmd(aFile)
@@ -268,7 +321,7 @@ def call_viewer_on_file(aFile = None, block=None):
 	viewer_cmd = get_viewer_cmd(mime_type, aFile)
 
 	if viewer_cmd is not None:
-		if gmShellAPI.run_command_in_shell(command=viewer_cmd, blocking=block):
+		if gmShellAPI.run_command_in_shell(command = viewer_cmd, blocking = block):
 			return True, ''
 
 	_log.warning("no viewer found via standard mailcap system")
@@ -313,15 +366,27 @@ def call_viewer_on_file(aFile = None, block=None):
 #		os.remove(file_to_display)
 
 	return True, ''
+
 #=======================================================================================
 if __name__ == "__main__":
 
-	if len(sys.argv) > 1 and sys.argv[1] == u'test':
+	if len(sys.argv) < 2:
+		sys.exit()
 
-		filename = sys.argv[2]
+	if sys.argv[1] != 'test':
+		sys.exit()
 
-		_get_system_startfile_cmd(filename)
-		print _system_startfile_cmd
-		#print guess_mimetype(filename)
-		#print get_viewer_cmd(guess_mimetype(filename), filename)
-		#print guess_ext_by_mimetype(mimetype=filename)
+	# for testing:
+	logging.basicConfig(level = logging.DEBUG)
+
+	filename = sys.argv[2]
+
+	#_get_system_startfile_cmd(filename)
+	#print(_system_startfile_cmd)
+	#print(guess_mimetype(filename))
+	#print(get_viewer_cmd(guess_mimetype(filename), filename))
+	#print(guess_ext_by_mimetype(mimetype=filename))
+	#call_viewer_on_file(aFile = filename, block=None)
+	status, desc = describe_file(filename)
+	print status
+	print desc
